﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;

using ICSharpCode.TextEditor;
using ICSharpCode.TextEditor.Actions;
using ICSharpCode.TextEditor.Document;

using Res = Hyperz.Mafia2Console.Properties.Resources;


namespace Hyperz.Mafia2Console
{
	public partial class MainForm : Form
	{
		[DllImport("MafiaDll.dll", EntryPoint = "InjectLua",
		   CharSet = CharSet.Ansi, ExactSpelling = true,
		   CallingConvention = CallingConvention.Cdecl)]
		internal static extern bool InjectLua(string lua);

		public MainForm()
		{
			InitializeComponent();

			ToolStripManager.VisualStylesEnabled = false;
		}

		private void DoEditAction(TextEditorControl editor, IEditAction action)
		{
			if (editor != null && action != null)
			{
				var area = editor.ActiveTextAreaControl.TextArea;

				editor.BeginUpdate();

				try
				{
					lock (editor.Document)
					{
						action.Execute(area);

						if (area.SelectionManager.HasSomethingSelected && area.AutoClearSelection)
						{
							if (area.Document.TextEditorProperties.DocumentSelectionMode == DocumentSelectionMode.Normal)
							{
								area.SelectionManager.ClearSelection();
							}
						}
					}
				}
				finally
				{
					editor.EndUpdate();
					area.Caret.UpdateCaretPosition();
				}
			}
		}

		private void SetStatus(string text, Image img = null)
		{
			this.lblStatusText.Text = text;
			this.lblStatusImg.Image = img;
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			var eventHandler = new EventHandler(UndoStack_Action);

			this.txtEditor.SetHighlighting("Lua");
			this.txtEditor.TextChanged += new EventHandler(txtEditor_TextChanged);
			this.txtEditor.ActiveTextAreaControl.SelectionManager.SelectionChanged += new EventHandler(SelectionManager_SelectionChanged);
			this.txtEditor.Document.UndoStack.ActionRedone += eventHandler;
			this.txtEditor.Document.UndoStack.ActionUndone += eventHandler;

			this.UndoStack_Action(null, null);
			this.SelectionManager_SelectionChanged(null, null);
			this.timer_Tick(null, null);
		}

		private void UndoStack_Action(object sender, EventArgs e)
		{
			var stack = this.txtEditor.Document.UndoStack;
			this.undoToolStripMenuItem.Enabled = stack.CanUndo;
			this.redoToolStripMenuItem.Enabled = stack.CanRedo;
		}

		private void SelectionManager_SelectionChanged(object sender, EventArgs e)
		{
			this.runSelectedCodeToolStripMenuItem.Enabled
				= this.menuRunSelectedCode.Enabled
				= this.txtEditor.ActiveTextAreaControl.SelectionManager.HasSomethingSelected;
		}

		private void txtEditor_TextChanged(object sender, EventArgs e)
		{
			this.UndoStack_Action(null, null);
		}

		private void openToolStripMenuItem_Click(object sender, EventArgs e)
		{
			DialogResult result = this.dlgOpen.ShowDialog();

			if (result == DialogResult.OK)
			{
				try
				{
					this.txtEditor.LoadFile(this.dlgOpen.FileName, false, true);
				}
				catch
				{
					MessageBox.Show("Could not open file.", "Error",
						MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
		}

		private void saveToolStripMenuItem_Click(object sender, EventArgs e)
		{
			if (!String.IsNullOrEmpty(this.txtEditor.FileName))
			{
				this.dlgSave.FileName = this.txtEditor.FileName;
			}

			DialogResult result = this.dlgSave.ShowDialog();

			if (result == DialogResult.OK)
			{
				try
				{
					this.txtEditor.SaveFile(this.dlgSave.FileName);
				}
				catch
				{
					MessageBox.Show("Could not save file.", "Error",
						MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
		}

		private void exitToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private void undoToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.DoEditAction(this.txtEditor, new Undo());
		}

		private void redoToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.DoEditAction(this.txtEditor, new Redo());
		}

		private void cutToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.DoEditAction(this.txtEditor, new Cut());
		}

		private void copyToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.DoEditAction(this.txtEditor, new Copy());
		}

		private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.DoEditAction(this.txtEditor, new Paste());
		}

		private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.DoEditAction(this.txtEditor, new SelectWholeDocument());
		}

		private void runCodeToolStripMenuItem_Click(object sender, EventArgs e)
		{
			string lua = this.txtEditor.Text;
			string time = DateTime.Now.ToLongTimeString();

			bool result = InjectLua(lua);

			if (result)
			{
				this.SetStatus("Code executed successfully [ " + time + " ]", Res.StatusOk);
			}
			else
			{
				this.SetStatus("Code execution failed [ " + time + " ]", Res.StatusFail);
			}

			this.lblStatusText.ToolTipText = lua;
		}

		private void runSelectedCodeToolStripMenuItem_Click(object sender, EventArgs e)
		{
			string lua = this.txtEditor.ActiveTextAreaControl.SelectionManager.SelectedText;
			string time = DateTime.Now.ToLongTimeString();

			bool result = InjectLua(lua);

			if (result)
			{
				this.SetStatus("Code executed successfully [ " + time + " ]", Res.StatusOk);
			}
			else
			{
				this.SetStatus("Code execution failed [ " + time + " ]", Res.StatusFail);
			}

			this.lblStatusText.ToolTipText = lua;
		}

		private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
		{
			// TODO: implement
		}

		private void timer_Tick(object sender, EventArgs e)
		{
			this.lblTime.Text = DateTime.Now.ToLongTimeString();
		}
	}
}
